require 'studio_game/player'
require 'studio_game/treasure_trove'

describe StudioGame::Player do
  
  before do
    @initial_health = 150
    @player = StudioGame::Player.new("larry", @initial_health)
    $stdout = StringIO.new
  end
  
  it "has a capitalized name" do
    @player.name.should == "Larry"
  end
  
  it "has an initial health" do  
    @player.health.should == 150
  end
  
  it "has a string representation" do
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))

    @player.to_s.should == "I'm Larry with health = 150, points = 100, and score = 250."
  end

  it "computes a score as the sum of its health and length of name" do    
    @player.score.should == 150
  end
  
  it "increases health by 15 when w00ted" do
    @player.w00t
    
    @player.health.should == @initial_health + 15
  end
  
  it "decreases health by 10 when blammed" do
    @player.blam
    
    @player.health.should == @initial_health - 10
  end

  it "computes points as the sum of all treasure points" do
    @player.points.should == 0
    
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))

    @player.points.should == 50

    @player.found_treasure(StudioGame::Treasure.new(:crowbar, 400))

    @player.points.should == 450

    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))

    @player.points.should == 500
  end

  it "computes a score as the sum of its health and points" do
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))

    @player.score.should == 250
  end

  it "yields each found treasure and its total points" do
    @player.found_treasure(StudioGame::Treasure.new(:skillet, 100))
    @player.found_treasure(StudioGame::Treasure.new(:skillet, 100))
    @player.found_treasure(StudioGame::Treasure.new(:hammer, 50))
    @player.found_treasure(StudioGame::Treasure.new(:bottle, 5))
    @player.found_treasure(StudioGame::Treasure.new(:bottle, 5))
    @player.found_treasure(StudioGame::Treasure.new(:bottle, 5))
    @player.found_treasure(StudioGame::Treasure.new(:bottle, 5))
    @player.found_treasure(StudioGame::Treasure.new(:bottle, 5))

    yielded = []
    @player.each_found_treasure do |treasure|
      yielded << treasure
    end

    yielded.should == [
      StudioGame::Treasure.new(:skillet, 200),
      StudioGame::Treasure.new(:hammer, 50),
      StudioGame::Treasure.new(:bottle, 25)
   ]
  end

  it "can be created from a CSV string" do
    player = StudioGame::Player.from_csv("larry,150")

    player.name.should == "Larry"
    player.health.should == 150
  end

  context "created with a default health" do
    before do
      @player = StudioGame::Player.new("larry")
    end
    
    it "has a health of 100" do
      @player.health.should == 100
    end
  end

  context "created with an intial health of 150" do
    before do
      @player = StudioGame::Player.new("larry", 150)
    end
    
    it "is strong" do
      expect(@player.strong?).to eq(true)
    end
  end


  context "created with an intial health of 100" do
    before do
      @player = StudioGame::Player.new("larry", 100)
    end
    
    it "is strong" do
      expect(@player.strong?).to eq(false)
    end
  end

  context "in a collection of players" do
    before do
      @player1 = StudioGame::Player.new("moe", 100)
      @player2 = StudioGame::Player.new("larry", 200)
      @player3 = StudioGame::Player.new("curly", 300)

      @players = [@player1, @player2, @player3]
    end

    it "is sorted by decreasing score" do
      @players.sort.should == [@player3, @player2, @player1]
    end
  end
end